var override_data = {
    etypes: {
        h: {cls: "load"},
        n: {cls: "node"},
        r: {cls: "refbus"},
        p: {cls: "gen"},
        c: {cls: "highlight"}
    },
    nodes: [
        {name: 0, type: "c", value: 0},
        {name: 1, type: "r", value: 0},
        {name: 2, type: "n", value: 0},
        {name: 3, type: "n", value: 0},
        {name: 4, type: "n", value: 0},
        {name: 5, type: "n", value: 0},
        {name: 6, type: "n", value: 0},
        {name: 7, type: "n", value: 0},
        {name: 8, type: "n", value: 0},
        {name: 9, type: "n", value: 0},
        {name: 10, type: "n", value: 0},
        {name: 11, type: "n", value: 0},
        {name: 12, type: "n", value: 0},
        {name: 13, type: "h", value: 0},
        {name: 14, type: "h", value: 0},
        {name: 15, type: "h", value: 0},
        {name: 16, type: "h", value: 0},
        {name: 17, type: "h", value: 0},
        {name: 18, type: "h", value: 0},
        {name: 19, type: "h", value: 0},
        {name: 20, type: "h", value: 0},
        {name: 21, type: "h", value: 0},
        {name: 22, type: "h", value: 0},
        {name: 23, type: "p", value: 0},
        {name: 24, type: "p", value: 0},
        {name: 25, type: "p", value: 0},
        {name: 26, type: "p", value: 0},
        {name: 27, type: "p", value: 0},
        {name: 28, type: "p", value: 0},
    ],
    links: [
        {source: 0, target: 1},
        {source: 1, target: 2},
        {source: 2, target: 3},
        {source: 3, target: 4},
        {source: 4, target: 5},
        {source: 2, target: 6},
        {source: 6, target: 7},
        {source: 7, target: 8},
        {source: 8, target: 9},
        {source: 6, target: 10},
        {source: 10, target: 11},
        {source: 11, target: 12},
        {source:  3, target: 13},
        {source:  4, target: 14},
        {source:  4, target: 23},
        {source:  5, target: 15},
        {source:  5, target: 24},
        {source:  6, target: 16},
        {source:  7, target: 17},
        {source:  8, target: 18},
        {source:  8, target: 25},
        {source:  9, target: 19},
        {source:  9, target: 26},
        {source: 10, target: 20},
        {source: 10, target: 27},
        {source: 11, target: 21},
        {source: 11, target: 28},
        {source: 12, target: 22},
    ]
}

